#include <stdio.h>
#include <windows.h>
#include <wincrypt.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>

void main(int argc, char *argv[])
{
        if (argc!=3)
        {
        	printf("Usage %s <infile> <outfile>\n", argv[0]);
        	return;
	}

	int hfi = open(argv[1], O_RDONLY | O_BINARY);
	if (hfi<0)
	{
		puts("Can't open input file !");
		return;
	}
	char magic[4];
	read(hfi, (void *)magic, 4);
	if (memcmp(magic, "KSX\0", 4))
	{
		puts("Not a KSX file !");
		close(hfi);
		return;
	}
	int hfo = open(argv[2], O_CREAT | O_TRUNC | O_RDWR | O_BINARY, S_IWRITE);
	if (hfo<0)
	{
		puts("Can't create output file !");
		close(hfi);
		return;
	}

	HCRYPTPROV hProv;
	if (!CryptAcquireContext(&hProv, 0, "Microsoft Enhanced Cryptographic Provider v1.0", 1, 0))
	{
		if (GetLastError()==NTE_BAD_KEYSET)
		{
			if (!CryptAcquireContext(&hProv, 0, "Microsoft Enhanced Cryptographic Provider v1.0", 1, 8))
			{
				puts("Can't acquire crypto context !");
				return;
			}
		}
	}

	int hdr_size = 0;
	read(hfi, (void *)&hdr_size, 4);
	printf("hdr size: %X\n", hdr_size);
	unsigned char *hdr = new unsigned char[hdr_size];
	read(hfi, hdr, hdr_size);

	//derive key
	HCRYPTHASH hHash;
	if (!CryptCreateHash(hProv, CALG_SHA1, 0, 0, &hHash))
	{
		puts("Can't create hash !");
		return;
	}
	if (!CryptHashData(hHash, hdr, hdr_size, 0))
	{
		puts("Can't hash hdr !");
		return;
	}		
	HCRYPTKEY hKey;
	if (!CryptDeriveKey(hProv, CALG_RC4, hHash, 0, &hKey))
	{
		puts("Can't derive key !");
		return;
	}
	CryptDestroyHash(hHash);
	delete hdr;
		
	int in_size = filelength(hfi)-hdr_size-4-4;;
	unsigned char *blk_buf = new unsigned char[1024];
	while(in_size)
	{
		DWORD chunk_sz = (in_size<1024) ? in_size : 1024;
		read(hfi, blk_buf, chunk_sz);
		in_size-=chunk_sz;
		if (!CryptDecrypt(hKey, 0, FALSE, 0, blk_buf, &chunk_sz))
		{
			puts("Decrypt failed !");
			return;
		}
		write(hfo, blk_buf, chunk_sz);
	}

	delete blk_buf;
	CryptDestroyKey(hKey);
	CryptReleaseContext(hProv, 0);
	close(hfo);
	close(hfi);
}
